@@ -1,10 +1,14 @@
 # -*- coding: utf-8 -*-
+import datetime
 import json
+from io import BytesIO
 
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import AnonymousUser
+from django.core.checks import Info
 from django.test import RequestFactory, TestCase
 from django.urls import reverse
+from openpyxl import load_workbook
 
 from wagtail.admin.edit_handlers import get_form_for_model
 from wagtail.admin.forms import WagtailAdminPageForm
@@ -37,9 +41,9 @@ class TestFormResponsesPanel(TestCase):
     def test_render_with_submissions(self):
         """Show the panel with the count of submission and a link to the list_submissions view."""
         self.client.post('/contact-us/', {
-            'your-email': 'bob@example.com',
-            'your-message': 'hello world',
-            'your-choices': {'foo': '', 'bar': '', 'baz': ''}
+            'your_email': 'bob@example.com',
+            'your_message': 'hello world',
+            'your_choices': {'foo': '', 'bar': '', 'baz': ''}
         })
 
         result = self.panel.render()
@@ -81,9 +85,9 @@ class TestFormResponsesPanelWithCustomSubmissionClass(TestCase):
             user=self.test_user,
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': 'email@domain.com',
-                'your-message': 'hi joe',
-                'your-choices': {'foo': '', 'bar': '', 'baz': ''}
+                'your_email': 'email@domain.com',
+                'your_message': 'hi joe',
+                'your_choices': {'foo': '', 'bar': '', 'baz': ''}
             }),
         )
         new_form_submission.submit_time = '2017-08-29T12:00:00.000Z'
@@ -219,9 +223,9 @@ class TestFormsSubmissionsList(TestCase, WagtailTestUtils):
         new_form_submission = FormSubmission.objects.create(
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "new@example.com",
-                'your-message': "this is a fairly new message",
-                'your-choices': ['foo', 'baz'],
+                'your_email': "new@example.com",
+                'your_message': "this is a fairly new message",
+                'your_choices': ['foo', 'baz'],
             }),
         )
         new_form_submission.submit_time = '2014-01-01T12:00:00.000Z'
@@ -230,8 +234,8 @@ class TestFormsSubmissionsList(TestCase, WagtailTestUtils):
         old_form_submission = FormSubmission.objects.create(
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "old@example.com",
-                'your-message': "this is a really old message",
+                'your_email': "old@example.com",
+                'your_message': "this is a really old message",
             }),
         )
         old_form_submission.submit_time = '2013-01-01T12:00:00.000Z'
@@ -379,6 +383,41 @@ class TestFormsSubmissionsList(TestCase, WagtailTestUtils):
         self.assertTrue('this is a really old message' in first_row_values)
 
 
+class TestFormsSubmissionsListLegacyFieldName(TestCase, WagtailTestUtils):
+    fixtures = ['test.json']
+
+    def setUp(self):
+        self.assertTrue(self.client.login(username='siteeditor', password='password'))
+        self.form_page = Page.objects.get(url_path='/home/contact-us-one-more-time/').specific
+
+        # running checks should show an info message AND update blank clean_name values
+
+        messages = FormFieldWithCustomSubmission.check()
+
+        self.assertEqual(
+            messages,
+            [Info('Added `clean_name` on 3 form field(s)', obj=FormFieldWithCustomSubmission)]
+        )
+
+        # check clean_name has been updated
+        self.assertEqual(
+            FormFieldWithCustomSubmission.objects.all()[0].clean_name,
+            'your-email'
+        )
+
+    def test_list_submissions(self):
+        response = self.client.get(reverse('wagtailforms:list_submissions', args=(self.form_page.id,)))
+
+        # Check response
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, 'wagtailforms/index_submissions.html')
+        self.assertEqual(len(response.context['data_rows']), 2)
+
+        # check display of list values within form submissions
+        self.assertContains(response, 'old@example.com')
+        self.assertContains(response, 'new@example.com')
+
+
 class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
     def setUp(self):
         # Create a form page
@@ -388,9 +427,9 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
         old_form_submission = FormSubmission.objects.create(
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "old@example.com",
-                'your-message': "this is a really old message",
-                'your-choices': ['foo', 'baz'],
+                'your_email': "old@example.com",
+                'your_message': "this is a really old message",
+                'your_choices': ['foo', 'baz'],
             }),
         )
         old_form_submission.submit_time = '2013-01-01T12:00:00.000Z'
@@ -399,8 +438,8 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
         new_form_submission = FormSubmission.objects.create(
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "new@example.com",
-                'your-message': "this is a fairly new message",
+                'your_email': "new@example.com",
+                'your_message': "this is a fairly new message",
             }),
         )
         new_form_submission.submit_time = '2014-01-01T12:00:00.000Z'
@@ -412,17 +451,32 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
     def test_list_submissions_csv_export(self):
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV'}
+            {'export': 'csv'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
+        data_lines = response.getvalue().decode().split("\n")
 
         self.assertEqual(data_lines[0], 'Submission date,Your email,Your message,Your choices\r')
         self.assertEqual(data_lines[1], '2013-01-01 12:00:00+00:00,old@example.com,this is a really old message,"foo, baz"\r')
         self.assertEqual(data_lines[2], '2014-01-01 12:00:00+00:00,new@example.com,this is a fairly new message,None\r')
 
+    def test_list_submissions_xlsx_export(self):
+        response = self.client.get(
+            reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
+            {'export': 'xlsx'}
+        )
+
+        self.assertEqual(response.status_code, 200)
+        workbook_data = response.getvalue()
+        worksheet = load_workbook(filename=BytesIO(workbook_data))['Sheet1']
+        cell_array = [[cell.value for cell in row] for row in worksheet.rows]
+        self.assertEqual(cell_array[0], ['Submission date', 'Your email', 'Your message', 'Your choices'])
+        self.assertEqual(cell_array[1], [datetime.datetime(2013, 1, 1, 12, 0), 'old@example.com', 'this is a really old message', 'foo, baz'])
+        self.assertEqual(cell_array[2], [datetime.datetime(2014, 1, 1, 12, 0), 'new@example.com', 'this is a fairly new message', 'None'])
+        self.assertEqual(len(cell_array), 3)
+
     def test_list_submissions_csv_large_export(self):
         for i in range(100):
             new_form_submission = FormSubmission.objects.create(
@@ -437,12 +491,12 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
 
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV'}
+            {'export': 'csv'}
         )
 
         # Check that csv export is not paginated
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
+        data_lines = response.getvalue().decode().split("\n")
         self.assertEqual(104, len(data_lines))
 
     def test_list_submissions_csv_export_after_filter_form_submissions_for_user_hook(self):
@@ -452,12 +506,12 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
 
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV'}
+            {'export': 'csv'}
         )
 
         # An user can export form submissions without the hook
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
+        data_lines = response.getvalue().decode().split("\n")
 
         self.assertEqual(data_lines[0], 'Submission date,Your email,Your message,Your choices\r')
         self.assertEqual(data_lines[1], '2013-01-01 12:00:00+00:00,old@example.com,this is a really old message,"foo, baz"\r')
@@ -466,7 +520,7 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
         with self.register_hook('filter_form_submissions_for_user', construct_forms_for_user):
             response = self.client.get(
                 reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-                {'action': 'CSV'}
+                {'export': 'csv'}
             )
 
         # An user can't export form submission with the hook
@@ -475,12 +529,12 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
     def test_list_submissions_csv_export_with_date_from_filtering(self):
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV', 'date_from': '01/01/2014'}
+            {'export': 'csv', 'date_from': '01/01/2014'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
+        data_lines = response.getvalue().decode().split("\n")
 
         self.assertEqual(data_lines[0], 'Submission date,Your email,Your message,Your choices\r')
         self.assertEqual(data_lines[1], '2014-01-01 12:00:00+00:00,new@example.com,this is a fairly new message,None\r')
@@ -488,12 +542,12 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
     def test_list_submissions_csv_export_with_date_to_filtering(self):
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV', 'date_to': '12/31/2013'}
+            {'export': 'csv', 'date_to': '12/31/2013'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
+        data_lines = response.getvalue().decode().split("\n")
 
         self.assertEqual(data_lines[0], 'Submission date,Your email,Your message,Your choices\r')
         self.assertEqual(data_lines[1], '2013-01-01 12:00:00+00:00,old@example.com,this is a really old message,"foo, baz"\r')
@@ -501,12 +555,12 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
     def test_list_submissions_csv_export_with_range_filtering(self):
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV', 'date_from': '12/31/2013', 'date_to': '01/02/2014'}
+            {'export': 'csv', 'date_from': '12/31/2013', 'date_to': '01/02/2014'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
+        data_lines = response.getvalue().decode().split("\n")
 
         self.assertEqual(data_lines[0], 'Submission date,Your email,Your message,Your choices\r')
         self.assertEqual(data_lines[1], '2014-01-01 12:00:00+00:00,new@example.com,this is a fairly new message,None\r')
@@ -515,8 +569,8 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
         unicode_form_submission = FormSubmission.objects.create(
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "unicode@example.com",
-                'your-message': 'こんにちは、世界',
+                'your_email': "unicode@example.com",
+                'your_message': 'こんにちは、世界',
             }),
         )
         unicode_form_submission.submit_time = '2014-01-02T12:00:00.000Z'
@@ -524,12 +578,12 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
 
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'date_from': '01/02/2014', 'action': 'CSV'}
+            {'date_from': '01/02/2014', 'export': 'csv'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_line = response.content.decode('utf-8').split("\n")[1]
+        data_line = response.getvalue().decode('utf-8').split("\n")[1]
         self.assertIn('こんにちは、世界', data_line)
 
     def test_list_submissions_csv_export_with_unicode_in_field(self):
@@ -545,9 +599,9 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
         unicode_form_submission = FormSubmission.objects.create(
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "unicode@example.com",
-                'your-message': "We don\'t need unicode here",
-                'vyberite-samuiu-liubimuiu-ide-dlia-razrabotke-na-python': "vim",
+                'your_email': "unicode@example.com",
+                'your_message': "We don\'t need unicode here",
+                'u0412u044bu0431u0435u0440u0438u0442u0435_u0441u0430u043cu0443u044e_u043bu044eu0431u0438u043cu0443u044e_ide_u0434u043bu044f_u0440u0430u0437u0440u0430u0431u043eu0442u043au0435_u043du0430_python': "vim",
             }),
         )
         unicode_form_submission.submit_time = '2014-01-02T12:00:00.000Z'
@@ -555,17 +609,18 @@ class TestFormsSubmissionsExport(TestCase, WagtailTestUtils):
 
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id, )),
-            {'date_from': '01/02/2014', 'action': 'CSV'}
+            {'date_from': '01/02/2014', 'export': 'csv'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
 
-        data_lines = response.content.decode('utf-8').split("\n")
+        data_lines = response.getvalue().decode('utf-8').split("\n")
         self.assertIn('Выберите самую любимую IDE для разработке на Python', data_lines[0])
         self.assertIn('vim', data_lines[1])
 
 
+
 class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
     def create_test_user_without_admin(self, username):
         return get_user_model().objects.create_user(username=username, password='123')
@@ -579,8 +634,8 @@ class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
             user=self.create_test_user_without_admin('user-john'),
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "old@example.com",
-                'your-message': "this is a really old message",
+                'your_email': "old@example.com",
+                'your_message': "this is a really old message",
             }),
         )
         old_form_submission.submit_time = '2013-01-01T12:00:00.000Z'
@@ -590,8 +645,8 @@ class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
             user=self.create_test_user_without_admin('user-m1kola'),
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "new@example.com",
-                'your-message': "this is a fairly new message",
+                'your_email': "new@example.com",
+                'your_message': "this is a fairly new message",
             }),
         )
         new_form_submission.submit_time = '2014-01-01T12:00:00.000Z'
@@ -603,12 +658,12 @@ class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
     def test_list_submissions_csv_export(self):
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV'}
+            {'export': 'csv'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
+        data_lines = response.getvalue().decode().split("\n")
 
         self.assertEqual(data_lines[0], 'Username,Submission date,Your email,Your message,Your choices\r')
         self.assertEqual(data_lines[1],
@@ -619,12 +674,12 @@ class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
     def test_list_submissions_csv_export_with_date_from_filtering(self):
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV', 'date_from': '01/01/2014'}
+            {'export': 'csv', 'date_from': '01/01/2014'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
+        data_lines = response.getvalue().decode().split("\n")
 
         self.assertEqual(data_lines[0], 'Username,Submission date,Your email,Your message,Your choices\r')
         self.assertEqual(data_lines[1],
@@ -633,12 +688,12 @@ class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
     def test_list_submissions_csv_export_with_date_to_filtering(self):
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV', 'date_to': '12/31/2013'}
+            {'export': 'csv', 'date_to': '12/31/2013'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
+        data_lines = response.getvalue().decode().split("\n")
 
         self.assertEqual(data_lines[0], 'Username,Submission date,Your email,Your message,Your choices\r')
         self.assertEqual(data_lines[1],
@@ -647,12 +702,12 @@ class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
     def test_list_submissions_csv_export_with_range_filtering(self):
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV', 'date_from': '12/31/2013', 'date_to': '01/02/2014'}
+            {'export': 'csv', 'date_from': '12/31/2013', 'date_to': '01/02/2014'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
+        data_lines = response.getvalue().decode().split("\n")
 
         self.assertEqual(data_lines[0], 'Username,Submission date,Your email,Your message,Your choices\r')
         self.assertEqual(data_lines[1],
@@ -663,8 +718,8 @@ class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
             user=self.create_test_user_without_admin('user-bob'),
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "unicode@example.com",
-                'your-message': 'こんにちは、世界',
+                'your_email': "unicode@example.com",
+                'your_message': 'こんにちは、世界',
             }),
         )
         unicode_form_submission.submit_time = '2014-01-02T12:00:00.000Z'
@@ -672,12 +727,12 @@ class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
 
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'date_from': '01/02/2014', 'action': 'CSV'}
+            {'date_from': '01/02/2014', 'export': 'csv'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_line = response.content.decode('utf-8').split("\n")[1]
+        data_line = response.getvalue().decode('utf-8').split("\n")[1]
         self.assertIn('こんにちは、世界', data_line)
 
     def test_list_submissions_csv_export_with_unicode_in_field(self):
@@ -696,7 +751,7 @@ class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
             form_data=json.dumps({
                 'your-email': "unicode@example.com",
                 'your-message': "We don\'t need unicode here",
-                'vyberite-samuiu-liubimuiu-ide-dlia-razrabotke-na-python': "vim",
+                'u0412u044bu0431u0435u0440u0438u0442u0435_u0441u0430u043cu0443u044e_u043bu044eu0431u0438u043cu0443u044e_ide_u0434u043bu044f_u0440u0430u0437u0440u0430u0431u043eu0442u043au0435_u043du0430_python': "vim",
             }),
         )
         unicode_form_submission.submit_time = '2014-01-02T12:00:00.000Z'
@@ -704,17 +759,18 @@ class TestCustomFormsSubmissionsExport(TestCase, WagtailTestUtils):
 
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'date_from': '01/02/2014', 'action': 'CSV'}
+            {'date_from': '01/02/2014', 'export': 'csv'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
 
-        data_lines = response.content.decode('utf-8').split("\n")
+        data_lines = response.getvalue().decode('utf-8').split("\n")
         self.assertIn('Выберите самую любимую IDE для разработке на Python', data_lines[0])
         self.assertIn('vim', data_lines[1])
 
 
+
 class TestCustomFormsSubmissionsList(TestCase, WagtailTestUtils):
     def create_test_user_without_admin(self, username):
         return get_user_model().objects.create_user(username=username, password='123')
@@ -728,8 +784,8 @@ class TestCustomFormsSubmissionsList(TestCase, WagtailTestUtils):
             user=self.create_test_user_without_admin('user-john'),
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "old@example.com",
-                'your-message': "this is a really old message",
+                'your_email': "old@example.com",
+                'your_message': "this is a really old message",
             }),
         )
         old_form_submission.submit_time = '2013-01-01T12:00:00.000Z'
@@ -739,8 +795,8 @@ class TestCustomFormsSubmissionsList(TestCase, WagtailTestUtils):
             user=self.create_test_user_without_admin('user-m1kola'),
             page=self.form_page,
             form_data=json.dumps({
-                'your-email': "new@example.com",
-                'your-message': "this is a fairly new message",
+                'your_email': "new@example.com",
+                'your_message': "this is a fairly new message",
             }),
         )
         new_form_submission.submit_time = '2014-01-01T12:00:00.000Z'
@@ -758,8 +814,8 @@ class TestCustomFormsSubmissionsList(TestCase, WagtailTestUtils):
                 user=self.create_test_user_without_admin('generated-username-%s' % i),
                 page=self.form_page,
                 form_data=json.dumps({
-                    'your-email': "generated-your-email-%s" % i,
-                    'your-message': "generated-your-message-%s" % i,
+                    'your_email': "generated-your-email-%s" % i,
+                    'your_message': "generated-your-message-%s" % i,
                 })
             )
             submission.save()
@@ -1039,10 +1095,10 @@ class TestFormsWithCustomSubmissionsList(TestCase, WagtailTestUtils):
             page=self.form_page,
             user=self.test_user_1,
             form_data=json.dumps({
-                'your-email': 'new@example.com',
+                'your_email': 'new@example.com',
                 'chocolate': 'White Chocolate',
                 'ingredients': 'White colouring',
-                'your-excitement': self.choices[2],
+                'your_excitement': self.choices[2],
             }),
         )
         new_form_submission.submit_time = '2017-10-01T12:00:00.000Z'
@@ -1052,10 +1108,10 @@ class TestFormsWithCustomSubmissionsList(TestCase, WagtailTestUtils):
             page=self.form_page,
             user=self.test_user_2,
             form_data=json.dumps({
-                'your-email': 'old@example.com',
+                'your_email': 'old@example.com',
                 'chocolate': 'Dark Chocolate',
                 'ingredients': 'Charcoal',
-                'your-excitement': self.choices[0],
+                'your_excitement': self.choices[0],
             }),
         )
         old_form_submission.submit_time = '2017-01-01T12:00:00.000Z'
@@ -1070,9 +1126,9 @@ class TestFormsWithCustomSubmissionsList(TestCase, WagtailTestUtils):
                 page=self.form_page,
                 user=self.test_user_1,
                 form_data=json.dumps({
-                    'your-email': "foo-%s@bar.com" % i,
+                    'your_email': "foo-%s@bar.com" % i,
                     'chocolate': 'Chocolate No.%s' % i,
-                    'your-excitement': self.choices[3],
+                    'your_excitement': self.choices[3],
                 }),
             )
             submission.save()
@@ -1106,13 +1162,13 @@ class TestFormsWithCustomSubmissionsList(TestCase, WagtailTestUtils):
     def test_list_submissions_csv_export(self):
         response = self.client.get(
             reverse('wagtailforms:list_submissions', args=(self.form_page.id,)),
-            {'action': 'CSV'}
+            {'export': 'csv'}
         )
 
         # Check response
         self.assertEqual(response.status_code, 200)
-        data_lines = response.content.decode().split("\n")
-        self.assertIn('filename=%s-export' % self.form_page.slug, response.get('Content-Disposition'))
+        data_lines = response.getvalue().decode().split("\n")
+        self.assertIn('filename="%s-export' % self.form_page.slug, response.get('Content-Disposition'))
         self.assertEqual(data_lines[0], 'Username,Submission date,Your email,Chocolate,Ingredients,Your Excitement\r')
         # first result should be the most recent as order_csv has been reversed
         self.assertEqual(data_lines[1], 'user-chocolate-maniac,2017-10-01 12:00:00+00:00,new@example.com,White Chocolate,White colouring,Much excitement\r')
@@ -1123,10 +1179,10 @@ class TestFormsWithCustomSubmissionsList(TestCase, WagtailTestUtils):
             page=self.form_page,
             user=self.create_test_user_without_admin('user-aaa-aaa'),
             form_data=json.dumps({
-                'your-email': 'new@example.com',
+                'your_email': 'new@example.com',
                 'chocolate': 'Old chocolate idea',
                 'ingredients': 'Sugar',
-                'your-excitement': self.choices[2],
+                'your_excitement': self.choices[2],
             }),
         )
         form_submission.submit_time = '2016-01-01T12:00:00.000Z'
@@ -1171,7 +1227,7 @@ class TestFormsWithCustomFormBuilderSubmissionsList(TestCase, WagtailTestUtils):
                 page=form_page,
                 form_data=json.dumps({
                     'name': 'John %s' % i,
-                    'device-ip-address': '192.0.2.%s' % i,
+                    'device_ip_address': '192.0.2.%s' % i,
                 }),
             )
             submission.save()
@@ -1193,7 +1249,12 @@ class TestFormsWithCustomFormBuilderSubmissionsList(TestCase, WagtailTestUtils):
         self.assertContains(response, '192.0.2.15')
 
 
-class TestIssue585(TestCase):
+class TestDuplicateFormFieldLabels(TestCase):
+    """
+    If a user creates two fields with the same label, data cannot be saved correctly.
+    See: https://github.com/wagtail/wagtail/issues/585
+    """
+
     fixtures = ['test.json']
 
     def setUp(self):
@@ -1231,3 +1292,38 @@ class TestIssue585(TestCase):
             response,
             text="There is another field with the label foo, please change one of them.",
         )
+
+    def test_adding_duplicate_form_labels_as_cleaned_name(self):
+        """
+        Ensure form submission fails when attempting to create labels that will resolve
+        to the same internal clean_name on the form field.
+        """
+
+        post_data = {
+            'title': "Form page!",
+            'content': "Some content",
+            'slug': 'contact-us',
+            'form_fields-TOTAL_FORMS': '3',
+            'form_fields-INITIAL_FORMS': '3',
+            'form_fields-MIN_NUM_FORMS': '0',
+            'form_fields-MAX_NUM_FORMS': '1000',
+            'form_fields-0-id': '',
+            'form_fields-0-label': 'LOW EARTH ORBIT',
+            'form_fields-0-field_type': 'singleline',
+            'form_fields-1-id': '',
+            'form_fields-1-label': 'low earth orbit',
+            'form_fields-1-field_type': 'singleline',
+            'form_fields-2-id': '',
+            'form_fields-2-label': 'bar',
+            'form_fields-2-field_type': 'singleline',
+        }
+        response = self.client.post(
+            reverse('wagtailadmin_pages:add', args=('tests', 'formpage', self.root_page.id)), post_data
+        )
+
+        self.assertEqual(response.status_code, 200)
+
+        self.assertContains(
+            response,
+            text="There is another field with the label LOW EARTH ORBIT, please change one of them.",
+        )
