@@ -1,10 +1,11 @@
 from django import forms
-from django.utils.translation import ugettext as _
-from django.utils.translation import ugettext_lazy
+from django.utils.crypto import constant_time_compare
+from django.utils.translation import gettext as _
+from django.utils.translation import gettext_lazy
 
 
 class PasswordViewRestrictionForm(forms.Form):
-    password = forms.CharField(label=ugettext_lazy("Password"), widget=forms.PasswordInput)
+    password = forms.CharField(label=gettext_lazy("Password"), widget=forms.PasswordInput)
     return_url = forms.CharField(widget=forms.HiddenInput)
 
     def __init__(self, *args, **kwargs):
@@ -13,7 +14,11 @@ class PasswordViewRestrictionForm(forms.Form):
 
     def clean_password(self):
         data = self.cleaned_data['password']
-        if data != self.restriction.password:
+        if not constant_time_compare(data, self.restriction.password):
             raise forms.ValidationError(_("The password you have entered is not correct. Please try again."))
 
         return data
+
+
+class TaskStateCommentForm(forms.Form):
+    comment = forms.CharField(label=_('Comment'), required=False, widget=forms.Textarea({'rows': 2}))
