@@ -1,34 +1,24 @@
 import hashlib
 import os.path
-import warnings
 from contextlib import contextmanager
 
 from django.conf import settings
 from django.db import models
 from django.dispatch import Signal
 from django.urls import reverse
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from taggit.managers import TaggableManager
 
 from wagtail.admin.models import get_object_usage
 from wagtail.core.models import CollectionMember
 from wagtail.search import index
 from wagtail.search.queryset import SearchableQuerySetMixin
-from wagtail.utils.deprecation import RemovedInWagtail210Warning
 
 
 class DocumentQuerySet(SearchableQuerySetMixin, models.QuerySet):
     pass
 
 
-def get_document_model():
-    warnings.warn("wagtail.documents.models.get_document_model "
-                  "has been moved to wagtail.documents.get_document_model",
-                  RemovedInWagtail210Warning)
-    from wagtail.documents import get_document_model
-    return get_document_model()
-
-
 class AbstractDocument(CollectionMember, index.Indexed, models.Model):
     title = models.CharField(max_length=255, verbose_name=_('title'))
     file = models.FileField(upload_to='documents', verbose_name=_('file'))
