@@ -196,6 +196,32 @@ class TestDocumentAddView(TestCase, WagtailTestUtils):
             evil_plans_collection
         )
 
+    @override_settings(WAGTAILDOCS_DOCUMENT_MODEL='tests.CustomDocument')
+    def test_unique_together_validation_error(self):
+        root_collection = Collection.get_first_root_node()
+        evil_plans_collection = root_collection.add_child(name="Evil plans")
+
+        # another document with a title to collide with
+        CustomDocument.objects.create(
+            title="Test document",
+            file=get_test_document_file(),
+            collection=evil_plans_collection
+        )
+
+        post_data = {
+            'title': "Test document",
+            'file': get_test_document_file(),
+            'collection': evil_plans_collection.id,
+        }
+        response = self.client.post(reverse('wagtaildocs:add'), post_data)
+
+        # Shouldn't redirect anywhere
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, 'wagtaildocs/documents/add.html')
+
+        # error message should be output on the page as a non-field error
+        self.assertContains(response, "Custom document with this Title and Collection already exists.")
+
 
 class TestDocumentAddViewWithLimitedCollectionPermissions(TestCase, WagtailTestUtils):
     def setUp(self):
@@ -417,6 +443,30 @@ class TestDocumentEditViewWithCustomDocumentModel(TestCase, WagtailTestUtils):
         # form media should be imported
         self.assertContains(response, 'wagtailadmin/js/draftail.js')
 
+    def test_unique_together_validation_error(self):
+        root_collection = Collection.get_first_root_node()
+        evil_plans_collection = root_collection.add_child(name="Evil plans")
+
+        # another document with a title to collide with
+        CustomDocument.objects.create(
+            title="Updated",
+            file=get_test_document_file(),
+            collection=evil_plans_collection
+        )
+
+        post_data = {
+            'title': "Updated",
+            'collection': evil_plans_collection.id,
+        }
+        response = self.client.post(reverse('wagtaildocs:edit', args=(self.document.id,)), post_data)
+
+        # Shouldn't redirect anywhere
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, 'wagtaildocs/documents/edit.html')
+
+        # error message should be output on the page as a non-field error
+        self.assertContains(response, "Custom document with this Title and Collection already exists.")
+
 
 class TestDocumentDeleteView(TestCase, WagtailTestUtils):
     def setUp(self):
@@ -935,6 +985,30 @@ class TestDocumentChooserUploadView(TestCase, WagtailTestUtils):
         # Document should be created
         self.assertTrue(models.Document.objects.filter(title="Test document").exists())
 
+    @override_settings(WAGTAILDOCS_DOCUMENT_MODEL='tests.CustomDocument')
+    def test_unique_together_validation(self):
+        root_collection = Collection.get_first_root_node()
+        evil_plans_collection = root_collection.add_child(name="Evil plans")
+        # another document with a title to collide with
+        CustomDocument.objects.create(
+            title="Test document",
+            file=get_test_document_file(),
+            collection=evil_plans_collection
+        )
+
+        response = self.client.post(reverse('wagtaildocs:chooser_upload'), {
+            'document-chooser-upload-title': "Test document",
+            'document-chooser-upload-file': get_test_document_file(),
+            'document-chooser-upload-collection': evil_plans_collection.id
+        })
+
+        # Shouldn't redirect anywhere
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, 'wagtaildocs/chooser/chooser.html')
+
+        # The form should have an error
+        self.assertContains(response, "Custom document with this Title and Collection already exists.")
+
 
 class TestDocumentChooserUploadViewWithLimitedPermissions(TestCase, WagtailTestUtils):
     def setUp(self):
