@@ -1,9 +1,8 @@
-from django.conf.urls import include, url
 from django.contrib.auth.models import Permission
 from django.contrib.contenttypes.models import ContentType
-from django.urls import reverse
+from django.urls import include, path, reverse
 from django.utils.html import format_html
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from wagtail.admin.menu import MenuItem
 from wagtail.core import hooks
@@ -15,7 +14,7 @@ from wagtail.snippets.permissions import user_can_edit_snippets
 @hooks.register('register_admin_urls')
 def register_admin_urls():
     return [
-        url(r'^snippets/', include(urls, namespace='wagtailsnippets')),
+        path('snippets/', include(urls, namespace='wagtailsnippets')),
     ]
 
 
@@ -29,7 +28,7 @@ def register_snippets_menu_item():
     return SnippetsMenuItem(
         _('Snippets'),
         reverse('wagtailsnippets:index'),
-        classnames='icon icon-snippet',
+        icon_name='snippet',
         order=500
     )
 
